@@ -7,6 +7,9 @@
 #define SOUP_SOCKET_H 1
 
 #include <libsoup/soup-types.h>
+#include <gio/gio.h>
+
+G_BEGIN_DECLS
 
 #define SOUP_TYPE_SOCKET            (soup_socket_get_type ())
 #define SOUP_SOCKET(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SOUP_TYPE_SOCKET, SoupSocket))
@@ -15,66 +18,63 @@
 #define SOUP_IS_SOCKET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((obj), SOUP_TYPE_SOCKET))
 #define SOUP_SOCKET_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), SOUP_TYPE_SOCKET, SoupSocketClass))
 
-typedef struct SoupSocketPrivate SoupSocketPrivate;
-
-struct SoupSocket {
+struct _SoupSocket {
 	GObject parent;
 
-	SoupSocketPrivate *priv;
 };
 
 typedef struct {
 	GObjectClass parent_class;
 
 	/* signals */
-	void (*connect_result) (SoupSocket *, guint);
 	void (*readable)       (SoupSocket *);
 	void (*writable)       (SoupSocket *);
 	void (*disconnected)   (SoupSocket *);
 
 	void (*new_connection) (SoupSocket *, SoupSocket *);
+
+	/* Padding for future expansion */
+	void (*_libsoup_reserved1) (void);
+	void (*_libsoup_reserved2) (void);
+	void (*_libsoup_reserved3) (void);
+	void (*_libsoup_reserved4) (void);
 } SoupSocketClass;
 
+#define SOUP_SOCKET_LOCAL_ADDRESS    "local-address"
+#define SOUP_SOCKET_REMOTE_ADDRESS   "remote-address"
 #define SOUP_SOCKET_FLAG_NONBLOCKING "non-blocking"
-#define SOUP_SOCKET_FLAG_NODELAY     "nodelay"
-#define SOUP_SOCKET_FLAG_REUSEADDR   "reuseaddr"
 #define SOUP_SOCKET_IS_SERVER        "is-server"
 #define SOUP_SOCKET_SSL_CREDENTIALS  "ssl-creds"
-
-GType soup_socket_get_type (void);
-
-SoupSocket    *soup_socket_new                (const char         *optname1,
-					       ...);
-
-guint          soup_socket_connect            (SoupSocket         *sock,
-					       SoupAddress        *remote_addr);
-gboolean       soup_socket_listen             (SoupSocket         *sock,
-					       SoupAddress        *local_addr);
-gboolean       soup_socket_start_ssl          (SoupSocket         *sock);
-
-void           soup_socket_disconnect         (SoupSocket         *sock);
-gboolean       soup_socket_is_connected       (SoupSocket         *sock);
+#define SOUP_SOCKET_ASYNC_CONTEXT    "async-context"
+#define SOUP_SOCKET_TIMEOUT	     "timeout"
 
 typedef void (*SoupSocketCallback)            (SoupSocket         *sock,
 					       guint               status,
 					       gpointer            user_data);
-typedef void (*SoupSocketListenerCallback)    (SoupSocket         *listener,
-					       SoupSocket         *sock,
-					       gpointer            user_data);
 
-SoupSocket    *soup_socket_client_new_async   (const char         *hostname,
-					       guint               port,
-					       gpointer            ssl_creds,
+GType soup_socket_get_type (void);
+
+SoupSocket    *soup_socket_new                (const char         *optname1,
+					       ...) G_GNUC_NULL_TERMINATED;
+
+void           soup_socket_connect_async      (SoupSocket         *sock,
+					       GCancellable       *cancellable,
 					       SoupSocketCallback  callback,
 					       gpointer            user_data);
-SoupSocket    *soup_socket_client_new_sync    (const char         *hostname,
-					       guint               port,
-					       gpointer            ssl_creds,
-					       guint              *status_ret);
-SoupSocket    *soup_socket_server_new         (SoupAddress        *local_addr,
-					       gpointer            ssl_creds,
-					       SoupSocketListenerCallback callback,
-					       gpointer            user_data);
+guint          soup_socket_connect_sync       (SoupSocket         *sock,
+					       GCancellable       *cancellable);
+
+gboolean       soup_socket_listen             (SoupSocket         *sock);
+
+gboolean       soup_socket_start_ssl          (SoupSocket         *sock,
+					       GCancellable       *cancellable);
+gboolean       soup_socket_start_proxy_ssl    (SoupSocket         *sock,
+					       const char         *ssl_host,
+					       GCancellable       *cancellable);
+gboolean       soup_socket_is_ssl             (SoupSocket         *sock);
+
+void           soup_socket_disconnect         (SoupSocket         *sock);
+gboolean       soup_socket_is_connected       (SoupSocket         *sock);
 
 SoupAddress   *soup_socket_get_local_address  (SoupSocket         *sock);
 SoupAddress   *soup_socket_get_remote_address (SoupSocket         *sock);
@@ -90,19 +90,26 @@ typedef enum {
 SoupSocketIOStatus  soup_socket_read       (SoupSocket         *sock,
 					    gpointer            buffer,
 					    gsize               len,
-					    gsize              *nread);
+					    gsize              *nread,
+					    GCancellable       *cancellable,
+					    GError            **error);
 SoupSocketIOStatus  soup_socket_read_until (SoupSocket         *sock,
 					    gpointer            buffer,
 					    gsize               len,
 					    gconstpointer       boundary,
 					    gsize               boundary_len,
 					    gsize              *nread,
-					    gboolean           *got_boundary);
+					    gboolean           *got_boundary,
+					    GCancellable       *cancellable,
+					    GError            **error);
 
 SoupSocketIOStatus  soup_socket_write      (SoupSocket         *sock,
 					    gconstpointer       buffer,
 					    gsize               len,
-					    gsize              *nwrote);
+					    gsize              *nwrote,
+					    GCancellable       *cancellable,
+					    GError            **error);
 
+G_END_DECLS
 
 #endif /* SOUP_SOCKET_H */
